'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _server = require('react-dom/server');

var _extractModules = require('./extractModules');

var _extractModules2 = _interopRequireDefault(_extractModules);

var _crossReferenceStats = require('./crossReferenceStats');

var _crossReferenceStats2 = _interopRequireDefault(_crossReferenceStats);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var AsyncRenderer = function () {
  function AsyncRenderer(element, context) {
    var _this = this;

    _classCallCheck(this, AsyncRenderer);

    this.modules = {};
    this.webpackModules = {};
    this.asyncRenderIdx = 0;
    this.asyncMountIdx = 0;
    this.asyncMountChildrenIdx = {};
    this.awaitForAsyncMount = 0;
    this.awaitForAsyncRender = 0;
    this.asyncRenderResults = {};
    this.asyncMountHasResults = [];
    this.asyncMountResults = {};
    this.renderPass = false;

    this.handleModuleLoaded = function () {
      if (_this.awaitForAsyncRender <= 0 && !_this.renderPass) {
        _this.renderPass = true;
        _this.asyncMountIdx = 0;
        _this.asyncMountChildrenIdx = {};
        _this.asyncRenderIdx = 0;
        (0, _server.renderToString)(_this.element);
        _this.handleAsyncComponentMounted();
      }
    };

    this.handleAsyncComponentMounted = function () {
      if (_this.awaitForAsyncMount <= 0 && _this.awaitForAsyncRender <= 0 && _this.renderPass) {
        _this.asyncMountIdx = 0;
        _this.asyncMountChildrenIdx = {};
        _this.asyncRenderIdx = 0;
        var markup = (0, _server.renderToString)(_this.element);
        if (_this.awaitForAsyncMount <= 0) {
          _this.resolve(markup);
        }
      }
    };

    this.getAsyncRenderIdx = function () {
      return _this.asyncRenderIdx++;
    };

    this.storeAsyncRenderResult = function (idx, Component) {
      _this.asyncRenderResults[idx] = Component;
    };

    this.getAsyncRenderResult = function (idx) {
      return _this.asyncRenderResults[idx] !== undefined ? _this.asyncRenderResults[idx] : null;
    };

    this.getAsyncMountIdx = function (parentIndex) {
      if (typeof parentIndex !== 'undefined' && parentIndex !== undefined && parentIndex !== null) {
        if (!_this.asyncMountChildrenIdx[parentIndex]) _this.asyncMountChildrenIdx[parentIndex] = 1;else _this.asyncMountChildrenIdx[parentIndex] = _this.asyncMountChildrenIdx[parentIndex] + 1;
        return parentIndex + '.' + _this.asyncMountChildrenIdx[parentIndex];
      }
      _this.asyncMountIdx = _this.asyncMountIdx + 1;
      return _this.asyncMountIdx + '';
    };

    this.storeAsyncMountResult = function (idx, data) {
      _this.asyncMountResults[idx] = data;
      _this.asyncMountHasResults.push(idx);
    };

    this.getAsyncMountResult = function (idx) {
      return _this.asyncMountResults[idx];
    };

    this.hasAsyncMountResult = function (idx) {
      return _this.asyncMountHasResults.indexOf(idx) !== -1;
    };

    this.addWebpackModules = function (name, modules) {
      var nextModules = {};
      for (var prop in modules) {
        if (modules.hasOwnProperty(prop)) {
          nextModules[prop] = _extends({}, modules[prop], { key: name });
        }
      }
      _this.webpackModules = _extends({}, _this.webpackModules, nextModules);
    };

    this.getInitialState = function () {
      return _extends({}, _this.asyncMountResults);
    };

    this.getChunks = function (stats, stats2) {
      stats = (0, _extractModules2.default)(stats, _this.modules, _this.webpackModules, stats2 ? true : false);
      if (stats2) {
        return (0, _crossReferenceStats2.default)(stats, stats2);
      }
      return stats;
    };

    this.getModules = function (stats, stats2) {
      var chunks = _this.getChunks(stats, stats2);

      var files = [].concat.apply([], chunks.map(function (module) {
        return module.files;
      })).filter(function (file) {
        return !file.match(/\.map$/);
      });
      var modules = [].concat.apply([], chunks.filter(function (module) {
        return module.key;
      }).map(function (module) {
        return {
          key: module.key,
          chunk: module.id,
          module: module.moduleId
        };
      }));

      return { files: files, modules: modules };
    };

    this.element = element;
    this.context = context;
    this.context.getInitialState = this.getInitialState;
    this.context.getChunks = this.getChunks;
    this.context.getModules = this.getModules;
  }

  _createClass(AsyncRenderer, [{
    key: 'render',
    value: function render(resolve, reject) {
      this.resolve = resolve;
      this.asyncMountIdx = 0;
      this.asyncMountChildrenIdx = {};
      var markup = (0, _server.renderToString)(this.element);
      if (!this.awaitForAsyncMount && !this.awaitForAsyncRender) {
        this.resolve(markup);
      } else if (!this.awaitForAsyncRender) {
        this.renderPass = true;
      }
    }
  }, {
    key: 'addModule',
    value: function addModule(name, path) {
      this.modules[name] = path;
    }
  }]);

  return AsyncRenderer;
}();

exports.default = AsyncRenderer;