'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.extractChunks = extractChunks;
exports.extractEntry = extractEntry;

var _validateStats = require('./utils/validateStats');

var _validateStats2 = _interopRequireDefault(_validateStats);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function extractChunks(stats, modules) {
  var chunkIds = [];
  var chunks = [];
  modules.forEach(function (module) {
    module.chunks.forEach(function (chunkId) {
      var chunk = stats.chunks.find(function (chunk) {
        return chunk.id === chunkId;
      });
      if (chunkIds.indexOf(chunk.id) === -1) {
        var nextChunk = _extends({}, module, chunk);
        nextChunk.moduleId = module.id;
        nextChunk.key = module.key;
        chunks.push(nextChunk);
      }
    });
  });
  return chunks;
}

function extractEntry(stats) {
  var chunkIds = [];
  var chunks = [];
  for (var prop in stats.entrypoints) {
    if (stats.entrypoints.hasOwnProperty(prop)) {
      stats.entrypoints[prop].chunks.forEach(function (chunkId) {
        if (chunkIds.indexOf(chunkId) === -1) {
          chunks.push(stats.chunks.find(function (chunk) {
            return chunk.id === chunkId;
          }));
          chunkIds.push(chunkId);
        }
      });
    }
  }
  return chunks;
}

exports.default = function (stats, modules, webpackModules) {
  var ignoreEntry = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

  (0, _validateStats2.default)(stats);

  var statModules = [];
  stats.modules.forEach(function (module) {
    var filter = false;
    var key = void 0;
    if (webpackModules[module.id]) {
      filter = true;
      key = webpackModules[module.id].key;
    } else {
      for (var prop in modules) {
        if (modules.hasOwnProperty(prop)) {
          if (module.identifier.indexOf(modules[prop]) === module.identifier.length - modules[prop].length) {
            filter = true;
            key = prop;
            break;
          }
        }
      }
    }

    if (filter) {
      statModules.push(_extends({}, module, {
        key: key
      }));
    }
    return false;
  });

  var chunks = extractChunks(stats, statModules);

  if (!ignoreEntry) {
    chunks.push.apply(chunks, _toConsumableArray(extractEntry(stats)));
  }

  return chunks;
};